@@ -84,7 +84,9 @@
 #include "postgres.h"
 
 #include "cubedata.h"
+#include "nodes/miscnodes.h"
 #include "utils/float.h"
+#include "varatt.h"
 
 /* All grammar constructs return strings */
 #define YYSTYPE char *
@@ -92,23 +94,19 @@
 /*
  * Bison doesn't allocate anything that needs to live across parser calls,
  * so we can easily have it use palloc instead of malloc.  This prevents
- * memory leaks if we error out during parsing.  Note this only works with
- * bison >= 2.0.  However, in bison 1.875 the default is to use alloca()
- * if possible, so there's not really much problem anyhow, at least if
- * you're building with gcc.
+ * memory leaks if we error out during parsing.
  */
 #define YYMALLOC palloc
 #define YYFREE   pfree
 
-static char *scanbuf;
-static int	scanbuflen;
-
 static int item_count(const char *s, char delim);
-static NDBOX *write_box(int dim, char *str1, char *str2);
-static NDBOX *write_point_as_box(int dim, char *str);
+static bool write_box(int dim, char *str1, char *str2,
+					  NDBOX **result, struct Node *escontext);
+static bool write_point_as_box(int dim, char *str,
+							   NDBOX **result, struct Node *escontext);
 
 
-#line 112 "cubeparse.c"
+#line 110 "cubeparse.c"
 
 # ifndef YY_CAST
 #  ifdef __cplusplus
@@ -131,49 +129,7 @@ static NDBOX *write_point_as_box(int dim, char *str);
 #  endif
 # endif
 
-
-/* Debug traces.  */
-#ifndef YYDEBUG
-# define YYDEBUG 0
-#endif
-#if YYDEBUG
-extern int cube_yydebug;
-#endif
-
-/* Token kinds.  */
-#ifndef YYTOKENTYPE
-# define YYTOKENTYPE
-  enum yytokentype
-  {
-    YYEMPTY = -2,
-    YYEOF = 0,                     /* "end of file"  */
-    YYerror = 256,                 /* error  */
-    YYUNDEF = 257,                 /* "invalid token"  */
-    CUBEFLOAT = 258,               /* CUBEFLOAT  */
-    O_PAREN = 259,                 /* O_PAREN  */
-    C_PAREN = 260,                 /* C_PAREN  */
-    O_BRACKET = 261,               /* O_BRACKET  */
-    C_BRACKET = 262,               /* C_BRACKET  */
-    COMMA = 263                    /* COMMA  */
-  };
-  typedef enum yytokentype yytoken_kind_t;
-#endif
-
-/* Value type.  */
-#if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
-typedef int YYSTYPE;
-# define YYSTYPE_IS_TRIVIAL 1
-# define YYSTYPE_IS_DECLARED 1
-#endif
-
-
-extern YYSTYPE cube_yylval;
-
-
-int cube_yyparse (NDBOX **result);
-
-
-
+#include "cubeparse.h"
 /* Symbol kind.  */
 enum yysymbol_kind_t
 {
@@ -577,7 +533,7 @@ static const yytype_int8 yytranslate[] =
 /* YYRLINE[YYN] -- Source line where rule number YYN was defined.  */
 static const yytype_uint8 yyrline[] =
 {
-       0,    46,    46,    73,   100,   118,   137,   141,   147,   153
+       0,    46,    46,    74,   102,   121,   141,   145,   151,   157
 };
 #endif
 
@@ -705,7 +661,7 @@ enum { YYENOMEM = -2 };
       }                                                           \
     else                                                          \
       {                                                           \
-        yyerror (result, YY_("syntax error: cannot back up")); \
+        yyerror (result, scanbuflen, escontext, YY_("syntax error: cannot back up")); \
         YYERROR;                                                  \
       }                                                           \
   while (0)
@@ -738,7 +694,7 @@ do {                                                                      \
     {                                                                     \
       YYFPRINTF (stderr, "%s ", Title);                                   \
       yy_symbol_print (stderr,                                            \
-                  Kind, Value, result); \
+                  Kind, Value, result, scanbuflen, escontext); \
       YYFPRINTF (stderr, "\n");                                           \
     }                                                                     \
 } while (0)
@@ -750,11 +706,13 @@ do {                                                                      \
 
 static void
 yy_symbol_value_print (FILE *yyo,
-                       yysymbol_kind_t yykind, YYSTYPE const * const yyvaluep, NDBOX **result)
+                       yysymbol_kind_t yykind, YYSTYPE const * const yyvaluep, NDBOX **result, Size scanbuflen, struct Node *escontext)
 {
   FILE *yyoutput = yyo;
   YY_USE (yyoutput);
   YY_USE (result);
+  YY_USE (scanbuflen);
+  YY_USE (escontext);
   if (!yyvaluep)
     return;
   YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
@@ -769,12 +727,12 @@ yy_symbol_value_print (FILE *yyo,
 
 static void
 yy_symbol_print (FILE *yyo,
-                 yysymbol_kind_t yykind, YYSTYPE const * const yyvaluep, NDBOX **result)
+                 yysymbol_kind_t yykind, YYSTYPE const * const yyvaluep, NDBOX **result, Size scanbuflen, struct Node *escontext)
 {
   YYFPRINTF (yyo, "%s %s (",
              yykind < YYNTOKENS ? "token" : "nterm", yysymbol_name (yykind));
 
-  yy_symbol_value_print (yyo, yykind, yyvaluep, result);
+  yy_symbol_value_print (yyo, yykind, yyvaluep, result, scanbuflen, escontext);
   YYFPRINTF (yyo, ")");
 }
 
@@ -808,7 +766,7 @@ do {                                                            \
 
 static void
 yy_reduce_print (yy_state_t *yyssp, YYSTYPE *yyvsp,
-                 int yyrule, NDBOX **result)
+                 int yyrule, NDBOX **result, Size scanbuflen, struct Node *escontext)
 {
   int yylno = yyrline[yyrule];
   int yynrhs = yyr2[yyrule];
@@ -821,7 +779,7 @@ yy_reduce_print (yy_state_t *yyssp, YYSTYPE *yyvsp,
       YYFPRINTF (stderr, "   $%d = ", yyi + 1);
       yy_symbol_print (stderr,
                        YY_ACCESSING_SYMBOL (+yyssp[yyi + 1 - yynrhs]),
-                       &yyvsp[(yyi + 1) - (yynrhs)], result);
+                       &yyvsp[(yyi + 1) - (yynrhs)], result, scanbuflen, escontext);
       YYFPRINTF (stderr, "\n");
     }
 }
@@ -829,7 +787,7 @@ yy_reduce_print (yy_state_t *yyssp, YYSTYPE *yyvsp,
 # define YY_REDUCE_PRINT(Rule)          \
 do {                                    \
   if (yydebug)                          \
-    yy_reduce_print (yyssp, yyvsp, Rule, result); \
+    yy_reduce_print (yyssp, yyvsp, Rule, result, scanbuflen, escontext); \
 } while (0)
 
 /* Nonzero means print parse trace.  It is left uninitialized so that
@@ -870,10 +828,12 @@ int yydebug;
 
 static void
 yydestruct (const char *yymsg,
-            yysymbol_kind_t yykind, YYSTYPE *yyvaluep, NDBOX **result)
+            yysymbol_kind_t yykind, YYSTYPE *yyvaluep, NDBOX **result, Size scanbuflen, struct Node *escontext)
 {
   YY_USE (yyvaluep);
   YY_USE (result);
+  YY_USE (scanbuflen);
+  YY_USE (escontext);
   if (!yymsg)
     yymsg = "Deleting";
   YY_SYMBOL_PRINT (yymsg, yykind, yyvaluep, yylocationp);
@@ -900,7 +860,7 @@ int yynerrs;
 `----------*/
 
 int
-yyparse (NDBOX **result)
+yyparse (NDBOX **result, Size scanbuflen, struct Node *escontext)
 {
     yy_state_fast_t yystate = 0;
     /* Number of tokens to shift before error messages enabled.  */
@@ -1149,7 +1109,7 @@ yyreduce:
 		dim = item_count(yyvsp[-3], ',');
 		if (item_count(yyvsp[-1], ',') != dim)
 		{
-			ereport(ERROR,
+			errsave(escontext,
 					(errcode(ERRCODE_INVALID_TEXT_REPRESENTATION),
 					 errmsg("invalid input syntax for cube"),
 					 errdetail("Different point dimensions in (%s) and (%s).",
@@ -1158,7 +1118,7 @@ yyreduce:
 		}
 		if (dim > CUBE_MAX_DIM)
 		{
-			ereport(ERROR,
+			errsave(escontext,
 					(errcode(ERRCODE_INVALID_TEXT_REPRESENTATION),
 					 errmsg("invalid input syntax for cube"),
 					 errdetail("A cube cannot have more than %d dimensions.",
@@ -1166,20 +1126,21 @@ yyreduce:
 			YYABORT;
 		}
 
-		*result = write_box( dim, yyvsp[-3], yyvsp[-1] );
+		if (!write_box(dim, yyvsp[-3], yyvsp[-1], result, escontext))
+			YYABORT;
 	}
-#line 1172 "cubeparse.c"
+#line 1133 "cubeparse.c"
     break;
 
   case 3: /* box: paren_list COMMA paren_list  */
-#line 74 "cubeparse.y"
+#line 75 "cubeparse.y"
         {
 		int			dim;
 
 		dim = item_count(yyvsp[-2], ',');
 		if (item_count(yyvsp[0], ',') != dim)
 		{
-			ereport(ERROR,
+			errsave(escontext,
 					(errcode(ERRCODE_INVALID_TEXT_REPRESENTATION),
 					 errmsg("invalid input syntax for cube"),
 					 errdetail("Different point dimensions in (%s) and (%s).",
@@ -1188,7 +1149,7 @@ yyreduce:
 		}
 		if (dim > CUBE_MAX_DIM)
 		{
-			ereport(ERROR,
+			errsave(escontext,
 					(errcode(ERRCODE_INVALID_TEXT_REPRESENTATION),
 					 errmsg("invalid input syntax for cube"),
 					 errdetail("A cube cannot have more than %d dimensions.",
@@ -1196,20 +1157,21 @@ yyreduce:
 			YYABORT;
 		}
 
-		*result = write_box( dim, yyvsp[-2], yyvsp[0] );
+		if (!write_box(dim, yyvsp[-2], yyvsp[0], result, escontext))
+			YYABORT;
 	}
-#line 1202 "cubeparse.c"
+#line 1164 "cubeparse.c"
     break;
 
   case 4: /* box: paren_list  */
-#line 101 "cubeparse.y"
+#line 103 "cubeparse.y"
         {
 		int			dim;
 
 		dim = item_count(yyvsp[0], ',');
 		if (dim > CUBE_MAX_DIM)
 		{
-			ereport(ERROR,
+			errsave(escontext,
 					(errcode(ERRCODE_INVALID_TEXT_REPRESENTATION),
 					 errmsg("invalid input syntax for cube"),
 					 errdetail("A cube cannot have more than %d dimensions.",
@@ -1217,20 +1179,21 @@ yyreduce:
 			YYABORT;
 		}
 
-		*result = write_point_as_box(dim, yyvsp[0]);
+		if (!write_point_as_box(dim, yyvsp[0], result, escontext))
+			YYABORT;
 	}
-#line 1223 "cubeparse.c"
+#line 1186 "cubeparse.c"
     break;
 
   case 5: /* box: list  */
-#line 119 "cubeparse.y"
+#line 122 "cubeparse.y"
         {
 		int			dim;
 
 		dim = item_count(yyvsp[0], ',');
 		if (dim > CUBE_MAX_DIM)
 		{
-			ereport(ERROR,
+			errsave(escontext,
 					(errcode(ERRCODE_INVALID_TEXT_REPRESENTATION),
 					 errmsg("invalid input syntax for cube"),
 					 errdetail("A cube cannot have more than %d dimensions.",
@@ -1238,49 +1201,50 @@ yyreduce:
 			YYABORT;
 		}
 
-		*result = write_point_as_box(dim, yyvsp[0]);
+		if (!write_point_as_box(dim, yyvsp[0], result, escontext))
+			YYABORT;
 	}
-#line 1244 "cubeparse.c"
+#line 1208 "cubeparse.c"
     break;
 
   case 6: /* paren_list: O_PAREN list C_PAREN  */
-#line 138 "cubeparse.y"
+#line 142 "cubeparse.y"
         {
 		yyval = yyvsp[-1];
 	}
-#line 1252 "cubeparse.c"
+#line 1216 "cubeparse.c"
     break;
 
   case 7: /* paren_list: O_PAREN C_PAREN  */
-#line 142 "cubeparse.y"
+#line 146 "cubeparse.y"
         {
 		yyval = pstrdup("");
 	}
-#line 1260 "cubeparse.c"
+#line 1224 "cubeparse.c"
     break;
 
   case 8: /* list: CUBEFLOAT  */
-#line 148 "cubeparse.y"
+#line 152 "cubeparse.y"
         {
 		/* alloc enough space to be sure whole list will fit */
 		yyval = palloc(scanbuflen + 1);
 		strcpy(yyval, yyvsp[0]);
 	}
-#line 1270 "cubeparse.c"
+#line 1234 "cubeparse.c"
     break;
 
   case 9: /* list: list COMMA CUBEFLOAT  */
-#line 154 "cubeparse.y"
+#line 158 "cubeparse.y"
         {
 		yyval = yyvsp[-2];
 		strcat(yyval, ",");
 		strcat(yyval, yyvsp[0]);
 	}
-#line 1280 "cubeparse.c"
+#line 1244 "cubeparse.c"
     break;
 
 
-#line 1284 "cubeparse.c"
+#line 1248 "cubeparse.c"
 
       default: break;
     }
@@ -1327,7 +1291,7 @@ yyerrlab:
   if (!yyerrstatus)
     {
       ++yynerrs;
-      yyerror (result, YY_("syntax error"));
+      yyerror (result, scanbuflen, escontext, YY_("syntax error"));
     }
 
   if (yyerrstatus == 3)
@@ -1344,7 +1308,7 @@ yyerrlab:
       else
         {
           yydestruct ("Error: discarding",
-                      yytoken, &yylval, result);
+                      yytoken, &yylval, result, scanbuflen, escontext);
           yychar = YYEMPTY;
         }
     }
@@ -1400,7 +1364,7 @@ yyerrlab1:
 
 
       yydestruct ("Error: popping",
-                  YY_ACCESSING_SYMBOL (yystate), yyvsp, result);
+                  YY_ACCESSING_SYMBOL (yystate), yyvsp, result, scanbuflen, escontext);
       YYPOPSTACK (1);
       yystate = *yyssp;
       YY_STACK_PRINT (yyss, yyssp);
@@ -1438,7 +1402,7 @@ yyabortlab:
 | yyexhaustedlab -- YYNOMEM (memory exhaustion) comes here.  |
 `-----------------------------------------------------------*/
 yyexhaustedlab:
-  yyerror (result, YY_("memory exhausted"));
+  yyerror (result, scanbuflen, escontext, YY_("memory exhausted"));
   yyresult = 2;
   goto yyreturnlab;
 
@@ -1453,7 +1417,7 @@ yyreturnlab:
          user semantic actions for why this is necessary.  */
       yytoken = YYTRANSLATE (yychar);
       yydestruct ("Cleanup: discarding lookahead",
-                  yytoken, &yylval, result);
+                  yytoken, &yylval, result, scanbuflen, escontext);
     }
   /* Do not reclaim the symbols of the rule whose action triggered
      this YYABORT or YYACCEPT.  */
@@ -1462,7 +1426,7 @@ yyreturnlab:
   while (yyssp != yyss)
     {
       yydestruct ("Cleanup: popping",
-                  YY_ACCESSING_SYMBOL (+*yyssp), yyvsp, result);
+                  YY_ACCESSING_SYMBOL (+*yyssp), yyvsp, result, scanbuflen, escontext);
       YYPOPSTACK (1);
     }
 #ifndef yyoverflow
@@ -1473,7 +1437,7 @@ yyreturnlab:
   return yyresult;
 }
 
-#line 161 "cubeparse.y"
+#line 165 "cubeparse.y"
 
 
 /* This assumes the string has been normalized by productions above */
@@ -1494,8 +1458,9 @@ item_count(const char *s, char delim)
 	return nitems;
 }
 
-static NDBOX *
-write_box(int dim, char *str1, char *str2)
+static bool
+write_box(int dim, char *str1, char *str2,
+		  NDBOX **result, struct Node *escontext)
 {
 	NDBOX	   *bp;
 	char	   *s;
@@ -1511,18 +1476,26 @@ write_box(int dim, char *str1, char *str2)
 	s = str1;
 	i = 0;
 	if (dim > 0)
-		bp->x[i++] = float8in_internal(s, &endptr, "cube", str1);
+	{
+		bp->x[i++] = float8in_internal(s, &endptr, "cube", str1, escontext);
+		if (SOFT_ERROR_OCCURRED(escontext))
+			return false;
+	}
 	while ((s = strchr(s, ',')) != NULL)
 	{
 		s++;
-		bp->x[i++] = float8in_internal(s, &endptr, "cube", str1);
+		bp->x[i++] = float8in_internal(s, &endptr, "cube", str1, escontext);
+		if (SOFT_ERROR_OCCURRED(escontext))
+			return false;
 	}
 	Assert(i == dim);
 
 	s = str2;
 	if (dim > 0)
 	{
-		bp->x[i] = float8in_internal(s, &endptr, "cube", str2);
+		bp->x[i] = float8in_internal(s, &endptr, "cube", str2, escontext);
+		if (SOFT_ERROR_OCCURRED(escontext))
+			return false;
 		/* code this way to do right thing with NaN */
 		point &= (bp->x[i] == bp->x[0]);
 		i++;
@@ -1530,7 +1503,9 @@ write_box(int dim, char *str1, char *str2)
 	while ((s = strchr(s, ',')) != NULL)
 	{
 		s++;
-		bp->x[i] = float8in_internal(s, &endptr, "cube", str2);
+		bp->x[i] = float8in_internal(s, &endptr, "cube", str2, escontext);
+		if (SOFT_ERROR_OCCURRED(escontext))
+			return false;
 		point &= (bp->x[i] == bp->x[i - dim]);
 		i++;
 	}
@@ -1550,11 +1525,13 @@ write_box(int dim, char *str1, char *str2)
 		SET_POINT_BIT(bp);
 	}
 
-	return bp;
+	*result = bp;
+	return true;
 }
 
-static NDBOX *
-write_point_as_box(int dim, char *str)
+static bool
+write_point_as_box(int dim, char *str,
+				   NDBOX **result, struct Node *escontext)
 {
 	NDBOX		*bp;
 	int			i,
@@ -1571,15 +1548,20 @@ write_point_as_box(int dim, char *str)
 	s = str;
 	i = 0;
 	if (dim > 0)
-		bp->x[i++] = float8in_internal(s, &endptr, "cube", str);
+	{
+		bp->x[i++] = float8in_internal(s, &endptr, "cube", str, escontext);
+		if (SOFT_ERROR_OCCURRED(escontext))
+			return false;
+	}
 	while ((s = strchr(s, ',')) != NULL)
 	{
 		s++;
-		bp->x[i++] = float8in_internal(s, &endptr, "cube", str);
+		bp->x[i++] = float8in_internal(s, &endptr, "cube", str, escontext);
+		if (SOFT_ERROR_OCCURRED(escontext))
+			return false;
 	}
 	Assert(i == dim);
 
-	return bp;
+	*result = bp;
+	return true;
 }
-
-#include "cubescan.c"
