@@ -11,6 +11,7 @@
 #include "libpq/pqformat.h"
 #include "ltree.h"
 #include "utils/memutils.h"
+#include "varatt.h"
 
 
 typedef struct
@@ -24,8 +25,8 @@ typedef struct
 #define LTPRS_WAITNAME	0
 #define LTPRS_WAITDELIM 1
 
-static void finish_nodeitem(nodeitem *lptr, const char *ptr,
-							bool is_lquery, int pos);
+static bool finish_nodeitem(nodeitem *lptr, const char *ptr,
+							bool is_lquery, int pos, struct Node *escontext);
 
 
 /*
@@ -33,7 +34,7 @@ static void finish_nodeitem(nodeitem *lptr, const char *ptr,
  * returns an ltree
  */
 static ltree *
-parse_ltree(const char *buf)
+parse_ltree(const char *buf, struct Node *escontext)
 {
 	const char *ptr;
 	nodeitem   *list,
@@ -46,7 +47,7 @@ parse_ltree(const char *buf)
 	int			charlen;
 	int			pos = 1;		/* character position for error messages */
 
-#define UNCHAR ereport(ERROR, \
+#define UNCHAR ereturn(escontext, NULL,\
 					   errcode(ERRCODE_SYNTAX_ERROR), \
 					   errmsg("ltree syntax error at character %d", \
 							  pos))
@@ -54,14 +55,14 @@ parse_ltree(const char *buf)
 	ptr = buf;
 	while (*ptr)
 	{
-		charlen = pg_mblen_cstr(ptr);
+		charlen = pg_mblen(ptr);
 		if (t_iseq(ptr, '.'))
 			num++;
 		ptr += charlen;
 	}
 
 	if (num + 1 > LTREE_MAX_LEVELS)
-		ereport(ERROR,
+		ereturn(escontext, NULL,
 				(errcode(ERRCODE_PROGRAM_LIMIT_EXCEEDED),
 				 errmsg("number of ltree labels (%d) exceeds the maximum allowed (%d)",
 						num + 1, LTREE_MAX_LEVELS)));
@@ -69,12 +70,12 @@ parse_ltree(const char *buf)
 	ptr = buf;
 	while (*ptr)
 	{
-		charlen = pg_mblen_cstr(ptr);
+		charlen = pg_mblen(ptr);
 
 		switch (state)
 		{
 			case LTPRS_WAITNAME:
-				if (ISALNUM(ptr))
+				if (ISLABEL(ptr))
 				{
 					lptr->start = ptr;
 					lptr->wlen = 0;
@@ -86,12 +87,13 @@ parse_ltree(const char *buf)
 			case LTPRS_WAITDELIM:
 				if (t_iseq(ptr, '.'))
 				{
-					finish_nodeitem(lptr, ptr, false, pos);
+					if (!finish_nodeitem(lptr, ptr, false, pos, escontext))
+						return NULL;
 					totallen += MAXALIGN(lptr->len + LEVEL_HDRSIZE);
 					lptr++;
 					state = LTPRS_WAITNAME;
 				}
-				else if (!ISALNUM(ptr))
+				else if (!ISLABEL(ptr))
 					UNCHAR;
 				break;
 			default:
@@ -105,12 +107,13 @@ parse_ltree(const char *buf)
 
 	if (state == LTPRS_WAITDELIM)
 	{
-		finish_nodeitem(lptr, ptr, false, pos);
+		if (!finish_nodeitem(lptr, ptr, false, pos, escontext))
+			return NULL;
 		totallen += MAXALIGN(lptr->len + LEVEL_HDRSIZE);
 		lptr++;
 	}
 	else if (!(state == LTPRS_WAITNAME && lptr == list))
-		ereport(ERROR,
+		ereturn(escontext, NULL,
 				(errcode(ERRCODE_SYNTAX_ERROR),
 				 errmsg("ltree syntax error"),
 				 errdetail("Unexpected end of input.")));
@@ -172,8 +175,12 @@ Datum
 ltree_in(PG_FUNCTION_ARGS)
 {
 	char	   *buf = (char *) PG_GETARG_POINTER(0);
+	ltree	   *res;
 
-	PG_RETURN_POINTER(parse_ltree(buf));
+	if ((res = parse_ltree(buf, fcinfo->context)) == NULL)
+		PG_RETURN_NULL();
+
+	PG_RETURN_POINTER(res);
 }
 
 PG_FUNCTION_INFO_V1(ltree_out);
@@ -232,7 +239,7 @@ ltree_recv(PG_FUNCTION_ARGS)
 		elog(ERROR, "unsupported ltree version number %d", version);
 
 	str = pq_getmsgtext(buf, buf->len - buf->cursor, &nbytes);
-	res = parse_ltree(str);
+	res = parse_ltree(str, NULL);
 	pfree(str);
 
 	PG_RETURN_POINTER(res);
@@ -259,7 +266,7 @@ ltree_recv(PG_FUNCTION_ARGS)
  * returns an lquery
  */
 static lquery *
-parse_lquery(const char *buf)
+parse_lquery(const char *buf, struct Node *escontext)
 {
 	const char *ptr;
 	int			num = 0,
@@ -277,7 +284,7 @@ parse_lquery(const char *buf)
 	int			charlen;
 	int			pos = 1;		/* character position for error messages */
 
-#define UNCHAR ereport(ERROR, \
+#define UNCHAR ereturn(escontext, NULL,\
 					   errcode(ERRCODE_SYNTAX_ERROR), \
 					   errmsg("lquery syntax error at character %d", \
 							  pos))
@@ -285,7 +292,7 @@ parse_lquery(const char *buf)
 	ptr = buf;
 	while (*ptr)
 	{
-		charlen = pg_mblen_cstr(ptr);
+		charlen = pg_mblen(ptr);
 
 		if (t_iseq(ptr, '.'))
 			num++;
@@ -297,7 +304,7 @@ parse_lquery(const char *buf)
 
 	num++;
 	if (num > LQUERY_MAX_LEVELS)
-		ereport(ERROR,
+		ereturn(escontext, NULL,
 				(errcode(ERRCODE_PROGRAM_LIMIT_EXCEEDED),
 				 errmsg("number of lquery items (%d) exceeds the maximum allowed (%d)",
 						num, LQUERY_MAX_LEVELS)));
@@ -305,12 +312,12 @@ parse_lquery(const char *buf)
 	ptr = buf;
 	while (*ptr)
 	{
-		charlen = pg_mblen_cstr(ptr);
+		charlen = pg_mblen(ptr);
 
 		switch (state)
 		{
 			case LQPRS_WAITLEVEL:
-				if (ISALNUM(ptr))
+				if (ISLABEL(ptr))
 				{
 					GETVAR(curqlevel) = lptr = (nodeitem *) palloc0(sizeof(nodeitem) * (numOR + 1));
 					lptr->start = ptr;
@@ -333,7 +340,7 @@ parse_lquery(const char *buf)
 					UNCHAR;
 				break;
 			case LQPRS_WAITVAR:
-				if (ISALNUM(ptr))
+				if (ISLABEL(ptr))
 				{
 					lptr++;
 					lptr->start = ptr;
@@ -361,22 +368,25 @@ parse_lquery(const char *buf)
 				}
 				else if (t_iseq(ptr, '|'))
 				{
-					finish_nodeitem(lptr, ptr, true, pos);
+					if (!finish_nodeitem(lptr, ptr, true, pos, escontext))
+						return NULL;
 					state = LQPRS_WAITVAR;
 				}
 				else if (t_iseq(ptr, '{'))
 				{
-					finish_nodeitem(lptr, ptr, true, pos);
+					if (!finish_nodeitem(lptr, ptr, true, pos, escontext))
+						return NULL;
 					curqlevel->flag |= LQL_COUNT;
 					state = LQPRS_WAITFNUM;
 				}
 				else if (t_iseq(ptr, '.'))
 				{
-					finish_nodeitem(lptr, ptr, true, pos);
+					if (!finish_nodeitem(lptr, ptr, true, pos, escontext))
+						return NULL;
 					state = LQPRS_WAITLEVEL;
 					curqlevel = NEXTLEV(curqlevel);
 				}
-				else if (ISALNUM(ptr))
+				else if (ISLABEL(ptr))
 				{
 					/* disallow more chars after a flag */
 					if (lptr->flag)
@@ -402,12 +412,12 @@ parse_lquery(const char *buf)
 			case LQPRS_WAITFNUM:
 				if (t_iseq(ptr, ','))
 					state = LQPRS_WAITSNUM;
-				else if (t_isdigit_cstr(ptr))
+				else if (t_isdigit(ptr))
 				{
 					int			low = atoi(ptr);
 
 					if (low < 0 || low > LTREE_MAX_LEVELS)
-						ereport(ERROR,
+						ereturn(escontext, NULL,
 								(errcode(ERRCODE_PROGRAM_LIMIT_EXCEEDED),
 								 errmsg("lquery syntax error"),
 								 errdetail("Low limit (%d) exceeds the maximum allowed (%d), at character %d.",
@@ -420,18 +430,18 @@ parse_lquery(const char *buf)
 					UNCHAR;
 				break;
 			case LQPRS_WAITSNUM:
-				if (t_isdigit_cstr(ptr))
+				if (t_isdigit(ptr))
 				{
 					int			high = atoi(ptr);
 
 					if (high < 0 || high > LTREE_MAX_LEVELS)
-						ereport(ERROR,
+						ereturn(escontext, NULL,
 								(errcode(ERRCODE_PROGRAM_LIMIT_EXCEEDED),
 								 errmsg("lquery syntax error"),
 								 errdetail("High limit (%d) exceeds the maximum allowed (%d), at character %d.",
 										   high, LTREE_MAX_LEVELS, pos)));
 					else if (curqlevel->low > high)
-						ereport(ERROR,
+						ereturn(escontext, NULL,
 								(errcode(ERRCODE_SYNTAX_ERROR),
 								 errmsg("lquery syntax error"),
 								 errdetail("Low limit (%d) is greater than high limit (%d), at character %d.",
@@ -451,7 +461,7 @@ parse_lquery(const char *buf)
 			case LQPRS_WAITCLOSE:
 				if (t_iseq(ptr, '}'))
 					state = LQPRS_WAITEND;
-				else if (!t_isdigit_cstr(ptr))
+				else if (!t_isdigit(ptr))
 					UNCHAR;
 				break;
 			case LQPRS_WAITND:
@@ -462,7 +472,7 @@ parse_lquery(const char *buf)
 				}
 				else if (t_iseq(ptr, ','))
 					state = LQPRS_WAITSNUM;
-				else if (!t_isdigit_cstr(ptr))
+				else if (!t_isdigit(ptr))
 					UNCHAR;
 				break;
 			case LQPRS_WAITEND:
@@ -485,11 +495,14 @@ parse_lquery(const char *buf)
 	}
 
 	if (state == LQPRS_WAITDELIM)
-		finish_nodeitem(lptr, ptr, true, pos);
+	{
+		if (!finish_nodeitem(lptr, ptr, true, pos, escontext))
+			return NULL;
+	}
 	else if (state == LQPRS_WAITOPEN)
 		curqlevel->high = LTREE_MAX_LEVELS;
 	else if (state != LQPRS_WAITEND)
-		ereport(ERROR,
+		ereturn(escontext, NULL,
 				(errcode(ERRCODE_SYNTAX_ERROR),
 				 errmsg("lquery syntax error"),
 				 errdetail("Unexpected end of input.")));
@@ -569,8 +582,9 @@ parse_lquery(const char *buf)
  * Close out parsing an ltree or lquery nodeitem:
  * compute the correct length, and complain if it's not OK
  */
-static void
-finish_nodeitem(nodeitem *lptr, const char *ptr, bool is_lquery, int pos)
+static bool
+finish_nodeitem(nodeitem *lptr, const char *ptr, bool is_lquery, int pos,
+				struct Node *escontext)
 {
 	if (is_lquery)
 	{
@@ -591,18 +605,19 @@ finish_nodeitem(nodeitem *lptr, const char *ptr, bool is_lquery, int pos)
 
 	/* Complain if it's empty or too long */
 	if (lptr->len == 0)
-		ereport(ERROR,
+		ereturn(escontext, false,
 				(errcode(ERRCODE_SYNTAX_ERROR),
 				 is_lquery ?
 				 errmsg("lquery syntax error at character %d", pos) :
 				 errmsg("ltree syntax error at character %d", pos),
 				 errdetail("Empty labels are not allowed.")));
 	if (lptr->wlen > LTREE_LABEL_MAX_CHARS)
-		ereport(ERROR,
+		ereturn(escontext, false,
 				(errcode(ERRCODE_NAME_TOO_LONG),
 				 errmsg("label string is too long"),
 				 errdetail("Label length is %d, must be at most %d, at character %d.",
 						   lptr->wlen, LTREE_LABEL_MAX_CHARS, pos)));
+	return true;
 }
 
 /*
@@ -730,8 +745,12 @@ Datum
 lquery_in(PG_FUNCTION_ARGS)
 {
 	char	   *buf = (char *) PG_GETARG_POINTER(0);
+	lquery	   *res;
 
-	PG_RETURN_POINTER(parse_lquery(buf));
+	if ((res = parse_lquery(buf, fcinfo->context)) == NULL)
+		PG_RETURN_NULL();
+
+	PG_RETURN_POINTER(res);
 }
 
 PG_FUNCTION_INFO_V1(lquery_out);
@@ -790,7 +809,7 @@ lquery_recv(PG_FUNCTION_ARGS)
 		elog(ERROR, "unsupported lquery version number %d", version);
 
 	str = pq_getmsgtext(buf, buf->len - buf->cursor, &nbytes);
-	res = parse_lquery(str);
+	res = parse_lquery(str, NULL);
 	pfree(str);
 
 	PG_RETURN_POINTER(res);
