@@ -1,5 +1,5 @@
 
-# Copyright (c) 2021-2022, PostgreSQL Global Development Group
+# Copyright (c) 2021-2023, PostgreSQL Global Development Group
 
 use strict;
 use warnings;
@@ -14,7 +14,7 @@ my $tempdir = PostgreSQL::Test::Utils::tempdir;
 
 my @walfiles = (
 	'00000001000000370000000C.gz', '00000001000000370000000D',
-	'00000001000000370000000E',    '00000001000000370000000F.partial',);
+	'00000001000000370000000E', '00000001000000370000000F.partial',);
 
 sub create_files
 {
@@ -57,8 +57,10 @@ command_fails_like(
 {
 	# like command_like but checking stderr
 	my $stderr;
-	my $result = IPC::Run::run [ 'pg_archivecleanup', '-d', '-n', $tempdir,
-		$walfiles[2] ], '2>', \$stderr;
+	my $result =
+	  IPC::Run::run [ 'pg_archivecleanup', '-d', '-n', $tempdir,
+		$walfiles[2] ],
+	  '2>', \$stderr;
 	ok($result, "pg_archivecleanup dry run: exit code 0");
 	like(
 		$stderr,
@@ -98,8 +100,8 @@ sub run_check
 	return;
 }
 
-run_check('',                 'pg_archivecleanup');
-run_check('.partial',         'pg_archivecleanup with .partial file');
+run_check('', 'pg_archivecleanup');
+run_check('.partial', 'pg_archivecleanup with .partial file');
 run_check('.00000020.backup', 'pg_archivecleanup with .backup file');
 
 done_testing();
