@@ -11,6 +11,8 @@ CREATE FUNCTION get_command_tag(pg_ddl_command)
   RETURNS text IMMUTABLE STRICT
   AS 'MODULE_PATHNAME' LANGUAGE C;
 
-CREATE FUNCTION get_altertable_subcmdtypes(pg_ddl_command)
-  RETURNS text[] IMMUTABLE STRICT
+CREATE FUNCTION get_altertable_subcmdinfo(IN cmd pg_ddl_command,
+    OUT cmdtype text,
+    OUT objdesc text)
+  RETURNS SETOF record IMMUTABLE STRICT
   AS 'MODULE_PATHNAME' LANGUAGE C;
